{ This demonstration program shows how to use the binary logic functions
  implemented in ChroMathsome.
  The reader may do with this code whatever he/she wants.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: 3 January '97

  Notes:
  ------
  All functions use (long-) strings as argument(s).
  This implies that you have to use the $H compiler directive (default value).
  Read the Long String Types section in the Delphi helpfile for more info.
  To extend the limitations (round-off errors! -> fuzzy logic results),
  decimals must also be represented as a string.
  BinLogic.dcu uses this string representation, rather than the actual number,
  as the argument(s) for its calculations.
  This way, the range is almost unlimited and the results are absolutely round-off error free!
  To check the validity of a number in a given base, you can use the function
  function IsValidNumber(B1: string; Base: NumberBase):boolean;
  (see example lower)
  Example: A string that represents a decimal number may only have characters ['0'...'9'].
  This is a valid representation of a decimal number:
  '123456789876543212345698745632145698712302102365478912345678987654321234569874563214569'
  These are invalid representations:
  '123456789E+12', '-123', '123.25', etc...
  The theoretical maximum length of a long string value is 2GB (two gigabytes).
  In practice this means that the maximum length of a long string value (as the
  representation of a decimal or any other binary type and thus the range) is limited
  only by the amount of memory available to an application.
  To keep things practical (and realistic), all functions require a wordsize
  ( a positive integer = cardinal).
  Any number (in the cardinal range) may be used as wordsize.
  If you enter a negative number as wordsize, the actual wordsize will be 0 meaning
  that ALL RESULTS WILL BE 0.
  If you set the wordsize equal to 0 (= do not care), BinLogic.dcu will use the
  default value (= 64) so that the maximum binary integer is a 64-bit number
  (= 1111...1111b (64 1's) = FFFFFFFFFFFFFFFFh = 18446744073709551615d
   = 1777777777777777777777o).
  Note that setting the wordsize to any value always implies that
  the binary representation (base 2) is a number with wordsize digits.
  Ex.:
  wordsize = 256 -> max. number = 1111...1111 (256 1's) = FFFF...FFFF (64 F's, not 256!)
  Also note that if your binary integer exceeds the current wordsize, the excess most
  significant (leading) bits are dropped (eventually, the result will be truncated).

  Read the BinLogic helpfile for more information about wordsizes and the implemented
  functions.



  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
                }

unit BinLog1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, BinLogic,
  StdCtrls;

type
  TForm1 = class(TForm)
    XORButton: TButton;
    ORButton: TButton;
    SRBitButton: TButton;
    SLBitButton: TButton;
    SLByteButton: TButton;
    Button8: TButton;
    SRByteButton: TButton;
    OctButton: TButton;
    BinButton: TButton;
    NOTButton: TButton;
    ANDButton: TButton;
    DecButton: TButton;
    HexButton: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    BaseLabel: TLabel;
    WordSizeLabel: TLabel;
    Edit3: TEdit;
    RLByteButton: TButton;
    RLBitButton: TButton;
    RRBitButton: TButton;
    RRByteButton: TButton;
    ASRButton: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure BaseClick(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure BinaryLogic(Sender: TObject);
    procedure BinaryShift(Sender: TObject);
    procedure Edit1Change(Sender: TObject);

  private
    { Private declarations }
    procedure ConvertNumbers(OldBase, NewBase: NumberBase);


  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  CurrentBase: NumberBase; // defined in MathUnit ...\BinLogic.DCU
  WordSize: cardinal;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
     CurrentBase:= Hex;
     WordSize:= 64;
     BaseLabel.Caption:= 'Current Base: HEX';
     WordSizeLabel.Caption:= 'Current WordSize: 64';
end;

procedure TForm1.ConvertNumbers(OldBase, NewBase: NumberBase);
var s1, s2: string;
begin
     s1:= Edit1.Text;
     s2:= Edit2.Text;
     case OldBase of
     Dec:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then Edit1.Text:= DecToBin(s1, WordSize);
             if s2 <> '' then Edit2.Text:= DecToBin(s2, WordSize);
             end;
         Oct:begin
             if s1 <> '' then Edit1.Text:= DecToOct(s1, WordSize);
             if s2 <> '' then Edit2.Text:= DecToOct(s2, WordSize);
             end;
         Hex:begin
             if s1 <> '' then Edit1.Text:= DecToHex(s1, WordSize);
             if s2 <> '' then Edit2.Text:= DecToHex(s2, WordSize);
             end;
         end; // case
         end;
     Bin:begin
         case NewBase of
         Dec:begin
             if s1 <> '' then Edit1.Text:= BinToDec(s1, WordSize);
             if s2 <> '' then Edit2.Text:= BinToDec(s2, WordSize);
             end;
         Oct:begin
             if s1 <> '' then Edit1.Text:= BinToOct(s1, WordSize);
             if s2 <> '' then Edit2.Text:= BinToOct(s2, WordSize);
             end;
         Hex:begin
             if s1 <> '' then Edit1.Text:= BinToHex(s1, WordSize);
             if s2 <> '' then Edit2.Text:= BinToHex(s2, WordSize);
             end;
         end; // case
         end;
     Oct:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then Edit1.Text:= OctToBin(s1, WordSize);
             if s2 <> '' then Edit2.Text:= OctToBin(s2, WordSize);
             end;
         Dec:begin
             if s1 <> '' then Edit1.Text:= OctToDec(s1, WordSize);
             if s2 <> '' then Edit2.Text:= OctToDec(s2, WordSize);
             end;
         Hex:begin
             if s1 <> '' then Edit1.Text:= OctToHex(s1, WordSize);
             if s2 <> '' then Edit2.Text:= OctToHex(s2, WordSize);
             end;
         end; // case
         end;
     Hex:begin
         case NewBase of
         Bin:begin
             if s1 <> '' then Edit1.Text:= HexToBin(s1, WordSize);
             if s2 <> '' then Edit2.Text:= HexToBin(s2, WordSize);
             end;
         Oct:begin
             if s1 <> '' then Edit1.Text:= HexToOct(s1, WordSize);
             if s2 <> '' then Edit2.Text:= HexToOct(s2, WordSize);
             end;
         Dec:begin
             if s1 <> '' then Edit1.Text:= HexToDec(s1, WordSize);
             if s2 <> '' then Edit2.Text:= HexToDec(s2, WordSize);
             end;
         end; // case
         end;
     end; // case
     Label1.Caption:= IntToStr(Length(Edit1.Text));
     Label2.Caption:= IntToStr(Length(Edit2.Text));
end;


procedure TForm1.BaseClick(Sender: TObject);
var OldBase: NumberBase;
begin
     OldBase:= CurrentBase;
     if Sender = HexButton then
     begin
     CurrentBase:= Hex;
     BaseLabel.Caption:= 'Current Base: HEX';
     if OldBase <> CurrentBase then
     ConvertNumbers(OldBase, CurrentBase);
     end
     else if Sender = DecButton then
     begin
     CurrentBase:= Dec;
     BaseLabel.Caption:= 'Current Base: DEC';
     if OldBase <> CurrentBase then
     ConvertNumbers(OldBase, CurrentBase);
     end
     else if Sender = BinButton then
     begin
     CurrentBase:= Bin;
     BaseLabel.Caption:= 'Current Base: BIN';
     if OldBase <> CurrentBase then
     ConvertNumbers(OldBase, CurrentBase);
     end
     else if Sender = OctButton then
     begin
     CurrentBase:= Oct;
     BaseLabel.Caption:= 'Current Base: OCT';
     if OldBase <> CurrentBase then
     ConvertNumbers(OldBase, CurrentBase);
     end

end;


{ Set the wordsize to a different value. }
procedure TForm1.Button8Click(Sender: TObject);
var i: cardinal;
begin
     i:= StrToInt(Edit3.Text);
     if i <> WordSize then
     begin
     WordSize:= i;
     WordSizeLabel.Caption:= 'Current WordSize: ' + IntToStr(WordSize);
     end;
end;

{ This procedure handles the binary logic functions.
  Note that the binary NOT is implemented in the next procedure <-> All these functions
  require 2 arguments.                                                  }
procedure TForm1.BinaryLogic(Sender: TObject);
var s1, s2: string;
begin
     s1:= Edit1.Text;
     s2:= Edit2.Text;
     if ((s1 = '') or (s2 = '')) then ShowMessage('Error: Too Few Arguments!')
     else
     begin
     if Sender = ORButton then Edit2.Text:= BinOR(s1, s2, CurrentBase, WordSize)
     else if Sender = ANDButton then Edit2.Text:= BinAND(s1, s2, CurrentBase, WordSize)
     else if Sender = XORButton then Edit2.Text:= BinXOR(s1, s2, CurrentBase, WordSize);
     // The following two lines give you an idea of how long the actual 'numbers' are.
     // If the wordsize > 64, they may be longer than the size of the editboxes...
     Label1.Caption:= IntToStr(Length(Edit1.Text));
     Label2.Caption:= IntToStr(Length(Edit2.Text));
     end;
end;


{ This procedure handles the binary rotation/shifting functions.
  Note that the binary NOT is also implemented here <-> All these functions
  require only 1 argument.                                                  }
procedure TForm1.BinaryShift(Sender: TObject);
var s1: string;
begin
     s1:= Edit1.Text;
     if (s1 = '') then ShowMessage('Error: Too Few Arguments!')
     else
     begin
     if Sender = SLBitButton then Edit2.Text:= BinShiftLeftBit(s1, CurrentBase, WordSize)
     else if Sender = SRBitButton then Edit2.Text:= BinShiftRightBit(s1, CurrentBase, WordSize)
     else if Sender = SLByteButton then Edit2.Text:= BinShiftLeftByte(s1, CurrentBase, WordSize)
     else if Sender = SRByteButton then Edit2.Text:= BinShiftRightByte(s1, CurrentBase, WordSize)
     else if Sender = RLBitButton then Edit2.Text:= BinRotateLeftBit(s1, CurrentBase, WordSize)
     else if Sender = RRBitButton then Edit2.Text:= BinRotateRightBit(s1, CurrentBase, WordSize)
     else if Sender = RLByteButton then Edit2.Text:= BinRotateLeftByte(s1, CurrentBase, WordSize)
     else if Sender = RRByteButton then Edit2.Text:= BinRotateRightByte(s1, CurrentBase, WordSize)
     else if Sender = NOTButton then Edit2.Text:= BinNot(s1, CurrentBase, WordSize)
     else if Sender = ASRButton then Edit2.Text:= ArithmeticShiftRightBit(s1, CurrentBase, WordSize);
     Label1.Caption:= IntToStr(Length(Edit1.Text));
     Label2.Caption:= IntToStr(Length(Edit2.Text));
     end;
end;

{ BinLogic has its own 'valid number checking' routine.
  For those who want to handle the exception their own way
  (i.e. prevent BinLogic from raising an exception) ...
  Here's a possible implementation for checking the validity of the user input.
  Note that, in this demonstration program, BinLogic handles the exception for invalid numbers
  entered in Edit2.  This way, you can test both ways ... }
procedure TForm1.Edit1Change(Sender: TObject);
begin
    if not (IsValidNumber(Edit1.Text, CurrentBase))
    then
    showmessage('''' + Edit1.Text + '''' + ' is not a valid number in the current base!');  
end;


end.
